import os
import re
import gzip
import random
import string
import shutil
from pathlib import Path
from typing import Literal, List
from urllib.parse import unquote

import pandas as pd
import dask.dataframe as dd
import dask.config

from .policy_defs import SCHEMA
from ..compute.backend import infer_cuda
from ..utils import as_path

def parse_line(line):
    try:
        ul = unquote(line).strip()
        ul = re.sub(r'[\n\t]','',ul)
        details,path = re.match(r'^[^|]+\|(.*)\| -- (.*)$', ul).groups()
        
        d = dict([re.match(r'([\w]+)=(.*)',l).groups() for l in details.split('|')])

        grp = re.match(r'(?:/data/user(?:/home)?/|/data/project/|/scratch/)([^/]+)',path)
        if grp:
            tld = grp.group(1)
        else:
            tld = None
            
        d.update({'path': path,
                  'tld': tld})
        return d
    except:
        return line

def convert(
        input_file: str | Path, 
        output_dir: str | Path | None = None, 
        output_name: str | Path | None = None
    ) -> None:
    """
    Converts a GPFS log file to parquet format. The data schema assumes the same policy definition from list-path-external and list-path-dirplus.

    Parameters
    ----------
    input_file : str | Path
        Path to the file to convert. Can be either compressed or uncompressed.
    output_dir : str | Path | None, optional
        Directory path in which to store the file. If the directory does not exist, it will be created. If None, the output directory will be set to ./input_file/../../parquet in accordance with our standard organization. By default None
    output_name : str | Path | None, optional
        Name of the output file. The name will be automatically appended with .parquet. If None, the name of the input file will be used. By default None
    """
    
    input_file = as_path(input_file)
    
    if output_dir is not None:
        output_dir = as_path(output_dir)
    else:
        output_dir = input_file.parent.parent.joinpath('parquet')
    
    if output_name is None:
        output_name = input_file.with_suffix('.parquet').name
    else:
        output_name = as_path(output_name).with_suffix('.parquet').name

    output_path = output_dir.joinpath(output_name)

    output_dir.mkdir(mode=0o2770, exist_ok=True)
    
    with gzip.open(input_file,'r') as f:
        dicts = [parse_line(l) for l in f]
    df = pd.DataFrame.from_dict(dicts).sort_values('tld')
    df = df.astype(SCHEMA)

    df.to_parquet(output_path,engine = 'pyarrow')


def hivize(
        parquet_path: str | Path, 
        hive_path: str | Path, 
        tld: str | List[str] | None = None,
        staging_path: str | Path | None = None,
        partition_size: str = '100MiB', 
        with_cuda: bool | Literal['infer'] = 'infer',
        **kwargs
    ) -> None:
    parquet_path = as_path(parquet_path)
    hive_path = as_path(hive_path)
    
    if staging_path is None:
        rand_str = ''.join(random.choices(string.ascii_letters + string.digits, k=8))
        staging_path = Path(os.getenv('TMPDIR')).joinpath(os.getenv('USER'),f'hive-{rand_str}')
        print(f"INFO: Using {staging_path} as temporary directory",flush=True)
    else:
        staging_path = as_path(staging_path)

    hive_path.mkdir(exist_ok=True,parents=True)
    staging_path.mkdir(exist_ok=True,parents=True)

    if with_cuda == 'infer':
        with_cuda = infer_cuda()

    if with_cuda:
        import dask_cudf as backend
        from dask_cudf.core import from_cudf as from_local
        dask.config.set({'dataframe.backend':'cudf'})
    else:
        import dask as backend
        from dask.dataframe import from_pandas as from_local
        dask.config.set({'dataframe.backend':'pandas'})

    def indexed_name(ind):
        return f"indexed-{ind}.parquet"

    if tld is not None:
        if not isinstance(tld,list):
            tld = [tld]
        predicates = [('tld','in',tld)]
    else:
        predicates = None
    print(f"DEBUG: Filtering predicates are: {predicates}",flush=True)

    acq = re.search(r'[\d]{4}-[\d]{2}-[\d]{2}',str(parquet_path)).group(0)
    print(f"DEBUG: Acquisition date is {acq}",flush=True)

    # The flat parquet is initially read in via dask to avoid reading the full dataset into memory which happens even 
    # when including predicates for filtering. The dask dataframe is converted to a regular dataframe to drastically 
    # improve indexing and sorting by removing partitions. The sorted dataframe is converted back to a dask dataframe 
    # to create partitions within the parquet dataset and write to multiple files defined by those partitions.
    ddf = backend.read_parquet(parquet_path,filters=predicates,columns = ['size','kballoc','access','create','modify','uid','gid','path','tld'])
    df = ddf.compute()
    df = df.set_index('path').sort_index().assign(acq=acq)
    ddf = from_local(df).repartition(partition_size=partition_size,force=True)
    ddf.to_parquet(staging_path,partition_on=['tld','acq'],name_function = indexed_name)

    shutil.copytree(staging_path,hive_path,dirs_exist_ok=True)
    shutil.rmtree(staging_path)
    pass