from pathlib import Path
import subprocess
from ..utils import parse_scontrol,as_path

__all__ = ['split','compress_logs']

def is_gz(filepath) -> bool:
    with open(filepath, 'rb') as f:
        return f.read(2) == b'\x1f\x8b'

def pigz_exists() -> bool:
    print("INFO: Checking for pigz")
    proc = subprocess.run('which pigz',shell=True,text=True,capture_output=True)
    
    if proc.returncode == 1:
        print("WARNING: pigz was not found on the PATH. Defaulting to slower zcat. Install pigz or add it to PATH to improve performance")
        return False
    else:
        print(f"INFO: pigz found at {proc.stdout.strip()}")
        return True

def split(log: str | Path, output_dir: str | Path | None = None, 
              lines: int = 5e6, prefix: str = 'list-', compress: bool = True,**kwargs) -> None:
    """
    Split a raw GPFS log file into smaller chunks. These chunks can be converted to parquet format to create a full parquet dataset for parallel analysis. Chunks are optionally recompressed after being split from the raw log.

    NOTE: pigz is the preferred tool for decompressing the raw log file due to its parallelization compared to gzip. pigz will be used as the default decompression tool if found on the PATH, otherwise gzip will be used. Install or make available pigz to improve splitting performance.

    Parameters
    ----------
    log : str | Path
        Path to the raw log file. Can be either compressed or uncompressed.
    output_dir : str | Path | None, optional
        Path to the output directory to store the chunks. If the directory does not exist, it will be created. If None, the path defaults to ./log/../../chunks in accordance with our standard organization scheme. By default None
    lines : int, optional
        Number of lines to split the log file by. By default 5e6
    prefix : str, optional
        String to prefix the individual chunk files with. Each file name will have the form '<prefix>XXX' where XXX is an incrementing index assigned by the 'split' command. By default 'list-'
    compress : bool, optional
        If True, compress each chunk using gzip. By default True
    """
    log = as_path(log)
    
    if output_dir is None:
        output_dir = log.parent.parent.joinpath('chunks')
    else:
        output_dir = as_path(output_dir)
    output_dir.mkdir(exist_ok=True,mode=0o2770)

    out = output_dir.joinpath(prefix)

    if is_gz(log):
        cat = 'pigz -dc' if pigz_exists() else 'zcat'
    else:
        cat = 'cat'
    
    split_cmd = f"{cat} {log} | split -a 3 -d -l {int(lines)} - {out}"

    subprocess.run(split_cmd,shell=True,text=True)

    if compress:
        compress_logs(output_dir)
    pass

def compress_logs(log_dir: str | Path) -> None:
    """
    Compress raw logs using gzip in parallel. If a directory contains both uncompressed and compressed logs, the compressed logs will be ignored.

    Parameters
    ----------
    log_dir : str | Path
        Path to the directory containing the raw logs.
    """
    nproc,_ = parse_scontrol()
    log_dir = as_path(log_dir)

    logs = [str(p) for p in log_dir.glob('*[!gz]')]
    log_str = '\n'.join(logs)

    print(f"INFO: {len(logs)} logs found. Beginning compression")
    
    zip_cmd = f"echo '{log_str}' | xargs -I {{}} -P {nproc} bash -c 'gzip {{}}'"
    subprocess.run(zip_cmd, shell=True)
    pass