from pathlib import Path
import pandas as pd
from ..compute import start_backend
from .utils import extract_run_date_from_filename
from .factory import get_aggregator
from typing import Literal, List
from typeguard import typechecked

__all__ = ['aggregate_gpfs_dataset']

def _check_dataset_path(dataset_path) -> Path:
    if not isinstance(dataset_path,Path):
        dataset_path = Path(dataset_path)

    if dataset_path.is_file():
        print(f"INFO: Found 1 file ({dataset_path})")
    elif dataset_path.is_dir():
        n_files = len(list(dataset_path.glob('*.parquet')))
        print(f"INFO: {n_files} parquet files found in dataset")
        
    if not dataset_path.exists():
        raise FileNotFoundError(f"{dataset_path} does not exist. Please check your input")
    
    return dataset_path

# ENH: The default naming scheme for the aggregated data needs to be refined. It should describe what aggregations were 
# performed such as including tld and cutoff date if both were specified
def _check_report_paths(
        report_dir: str | Path | None,
        report_name: str | Path | None,
        parent_path: Path
        ) -> Path:
    if report_dir is None:
        report_dir = parent_path.joinpath('reports')
    elif not isinstance(report_dir,Path):
        report_dir = Path(report_dir)
    
    report_dir.mkdir(exist_ok=True,mode=0o2770)
    
    if report_name is None:
        report_name = 'tld_agg.parquet'
    
    report_path = report_dir.joinpath(report_name)
    return report_path

def _check_timedelta_values(vals,unit):
    if (vals is None) != (unit is None):
        raise ValueError("Must specify either both or neither of delta_vals and delta_unit")
    pass

# ENH: Refactor this, it should not automatically create the compute backend, that should be passed to it. Creating the 
# backend is beyond the scope of the process module and this function specifically. Can wrap this with the backend 
# creation in a CLI command later for convenience, but that shouldn't be baked in

@typechecked
def aggregate_gpfs_dataset(
    dataset_path: str | Path,
    run_date: pd.Timestamp | None = None,
    delta_vals: int | List[int] | None = None,
    delta_unit: Literal['D','W','M','Y'] | None = None,
    time_val: Literal['access','modify','create'] = 'access',
    report_dir: str | Path | None = None,
    report_name: str | Path | None = None,
    n_workers: int | None = None,
    with_cuda: Literal['infer'] | bool = 'infer',
    with_dask: Literal['infer'] | bool = 'infer',
    **kwargs
) -> None:
    # Input checking
    dataset_path = _check_dataset_path(dataset_path)
    if dataset_path.is_file():
        parent_path = dataset_path.parent.parent
    else:
        parent_path = dataset_path.parent

    report_path = _check_report_paths(report_dir,report_name,parent_path)
    _check_timedelta_values(delta_vals,delta_unit)

    if run_date is None:
        run_date = extract_run_date_from_filename(dataset_path)

    manager,backend = start_backend(
        dataset_path=dataset_path,
        with_cuda=with_cuda,
        with_dask=with_dask,
        n_workers=n_workers,
        **kwargs
        )
    
    try:
        aggregator = get_aggregator(backend)
        df = aggregator.read_parquet(dataset_path)
        grps = ['tld']
        if delta_vals is not None:
            cutoffs = aggregator.create_timedelta_cutoffs(delta_vals,delta_unit,run_date)
            labels  = aggregator.create_timedelta_labels(delta_vals,delta_unit)
            df['dt_grp'] = aggregator.cut_dt(df[time_val],cutoffs,labels)
            grps.append('dt_grp')
        else:
            cutoffs, labels = [None,None]
        
        df_agg = aggregator.aggregate(df, col = ['size'], grps = grps, funcs = ['sum','count'])
        df_agg = df_agg.rename(columns={'sum':'bytes','count':'file_count'})
        df_agg.to_parquet(report_path)
    finally:
        if manager is not None:
            manager.close()
    
    return None